/*
 * Project name:
     tRF click (Using mikroE's tRF click board)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
     20140416:
       - initial release (FJ);
 * Description:
      A simple example of using tRF click board, which will send the text string
      via the Telit's LE70-868 RF module when PORTD.B0 button is pressed and
      display the received string on the Lcd.
 * Test configuration:
     MCU:             STM32F107VC
                      http://www.st.com/internet/com/TECHNICAL_RESOURCES/TECHNICAL_LITERATURE/DATASHEET/CD00220364.pdf
     Dev.Board:       EasyMx PRO v7 for STM32
                      http://www.mikroe.com/eng/products/view/852/easymx-pro-v7-for-stm32/
     Oscillator:      HS-PLL 72.0000 MHz, internal 8.0000 MHz RC
     Ext. Modules:    tRF click board - ac:tRF_click
                      2x16 lcd character display - ac:Lcd
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
 * NOTES:
     - Place tRF click board in the mikroBUS socket 1 on the EasyMx Pro v7 board.
     - Pull-down PORTD.B0, Pull-up SW10.7 on the EasyMx Pro v7 board.
 */

// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

char rxBuff[17];                         // Buffer string for storing data
char rxCnt = 0;                          // Counter for data writen in buffer string
unsigned short Received = 0;             // Received data flag
char lab1[17] = {0};

void updateLabel(char *newText, int x, int y, char* lbl){
  TFT_Set_Font(&TFT_defaultFont, CL_WHITE, FO_HORIZONTAL); // Color of background
  TFT_Write_Text(lbl, x, y);
  strcpy(lbl, newText);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL); // Color of text
  TFT_Write_Text(lbl, x, y);
  delay_ms(50);
}

void DrawFrame() {
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_BLACK, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("tRF  Click  Board  Demo", 25, 14);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMx Pro v7", 19, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLUE, FO_HORIZONTAL);
  TFT_Write_Text("Send string with PD0", 40, 140);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("Recieved string:", 40, 80);
}

// Interrupt routine
void interrupt() iv IVT_INT_USART3 ics ICS_AUTO {
  char tmp;                              // Temporary variable declaration
    tmp = UART3_Read();                  // Store received data in the temporary variable
    if ((tmp == '\r') || (rxCnt > 15)) { // Check if data counter is larger than the buffer string size
                                         // or the received data is equal to the Carriage Return character
      rxBuff[rxCnt] = 0;                 // Terminate buffer string
      rxCnt = 0;                         // Reset data counter
      Received = 1;                      // Set Received data flag
    }
    else{
      rxBuff[rxCnt] = tmp;               // Populate buffer string with received data
      rxCnt = rxCnt + 1;                 // Increase data counter
    }
}

// MCU initialization
void MCU_Init(){
  // Set Buttons direction
  GPIO_Digital_Input(&GPIOD_IDR, _GPIO_PINMASK_0);

  TFT_Set_Default_Mode();
  TFT_Init_ILI9341_8bit(320, 240);   // Initialize TFT display
  DrawFrame();
  
  // Initialize UART module at 19200 bps
  UART3_Init_Advanced(19200, _UART_8_BIT_DATA, _UART_NOPARITY, _UART_ONE_STOPBIT, &_GPIO_MODULE_USART3_PD89);
  Delay_ms(100);                     // Wait for UART module to stabilize
  
  // Enable Usart Receiver interrupt:
  USART3_CR1bits.RXNEIE = 1;         // enable uart rx interrupt
  NVIC_IntEnable(IVT_INT_USART3);    // enable interrupt vector
  Enableinterrupts();
}

void main() {
  MCU_Init();                            // Initialize the MCU
  while(1) {
    if (Button(&GPIOD_IDR, 0, 1, 1))  {  // If PORTD.B0 is pressed :
      UART3_Write_Text("EchoTest\r");    // send text string
      Delay_ms(250);                     // wait for 250ms
    }
    
    if(Received){                        // If data is received via UART :
      updateLabel(rxBUFF, 150, 80, lab1);
      Received = 0;                      // clear received data flag
      Delay_ms(250);                     // wait for 250ms
    }
  }
}